.AssignmentComponent {
  color: var(--text-color);
}

.collapse-header {
  color: var(--text-color);
  font-size: 12pt;
  margin: 8pt;
  user-select: none;
  -webkit-user-select: none;
  cursor: pointer;

  /* border-top: 1px solid var(--text-color); */
}

.collapse-header:hover .flavor-icon {
  color: var(--accent-color);
}

.collapse-body {
  margin-left: 10pt;
}

.collapse-body ul {
  list-style-type: none;
}

.collapse-icon {
  float: right;
  color: var(--text-color);
  transition: transform .2s cubic-bezier(.4,1,.75,.9), -webkit-transform .2s cubic-bezier(.4,1,.75,.9);
}

.collapse-icon-open {
  -webkit-transform: rotate(180deg);
  transform: rotate(180deg);
}

.flavor-icon {
  color: var(--text-color);
  padding-right: 6pt;
  transition: 0.2s;
}